@@ -15,37 +15,73 @@ use alloc::vec::Vec;
 /// ```rust
 /// use tinyvec::*;
 ///
-/// let empty_av = tiny_vec!([u8; 16]);
-///
+/// // The backing array type can be specified in the macro call
+/// let empty_tv = tiny_vec!([u8; 16]);
 /// let some_ints = tiny_vec!([i32; 4], 1, 2, 3);
+/// let many_ints = tiny_vec!([i32; 4], 1, 2, 3, 4, 5, 6, 7, 8, 9, 10);
+///
+/// // Or left to inference
+/// let empty_tv: TinyVec<[u8; 16]> = tiny_vec!();
+/// let some_ints: TinyVec<[i32; 4]> = tiny_vec!(1, 2, 3);
+/// let many_ints: TinyVec<[i32; 4]> = tiny_vec!(1, 2, 3, 4, 5, 6, 7, 8, 9, 10);
 /// ```
 #[macro_export]
 macro_rules! tiny_vec {
   ($array_type:ty) => {
     {
-      let mut tv: TinyVec<$array_type> = Default::default();
+      let mut tv: $crate::TinyVec<$array_type> = Default::default();
       tv
     }
   };
-  ($array_type:ty, $($elem:expr),*) => {
+  ($array_type:ty, $($elem:expr),* $(,)?) => {
     {
-      let mut tv: TinyVec<$array_type> = Default::default();
-      $( tv.push($elem); )*
-      tv
+      // Note(Lokathor): This goofy looking thing will count the number of
+      // `$elem` entries we were given. We can't spit out the "+1"s on their
+      // own, we need to use `$elem` in the repetition-expansion somehow.
+      // However, we also can't assume it's `Copy` data, so we must use `$elem`
+      // only once "for real" in the expansion as a whole. To achieve this, we
+      // can `stringify!` each element in an inner block, then have the block
+      // return a 1. The stringification is a compile time thing, it won't
+      // actually move any values.
+      const INVOKED_ELEM_COUNT: usize = 0 $( + { let _ = stringify!($elem); 1 })*;
+      // If we have more `$elem` than the `CAPACITY` we will simply go directly
+      // to constructing on the heap.
+      let av: $crate::TinyVec<$array_type> = $crate::TinyVec::from_either_with_capacity(
+        INVOKED_ELEM_COUNT,
+        #[inline(always)] || $crate::array_vec!($array_type, $($elem),*),
+        #[inline(always)] || vec!($($elem),*));
+      av
     }
   };
+  () => {
+    tiny_vec!(_)
+  };
+  ($($elem:expr),*) => {
+    tiny_vec!(_, $($elem),*)
+  };
 }
 
 /// A vector that starts inline, but can automatically move to the heap.
 ///
 /// * Requires the `alloc` feature
 ///
-/// Note: This is currently an enum but you can mostly ignore that detail if you
-/// like. In future versions this **may** become an opaque struct, but for now
-/// we're being a little more liberal about allowing you to "access the
-/// internals" since no safety invariants are on the line. It's kinda wild how
-/// much you can just let people poke at stuff without worry when it's 100% safe
-/// code.
+/// A `TinyVec` is either an Inline([`ArrayVec`](crate::ArrayVec::<A>)) or
+/// Heap([`Vec`](https://doc.rust-lang.org/alloc/vec/struct.Vec.html)). The
+/// interface for the type as a whole is a bunch of methods that just match on
+/// the enum variant and then call the same method on the inner vec.
+///
+/// ## Construction
+///
+/// Because it's an enum, you can construct a `TinyVec` simply by making an
+/// `ArrayVec` or `Vec` and then putting it into the enum.
+///
+/// There is also a macro
+///
+/// ```rust
+/// # use tinyvec::*;
+/// let empty_tv = tiny_vec!([u8; 16]);
+/// let some_ints = tiny_vec!([i32; 4], 1, 2, 3);
+/// ```
 #[derive(Clone)]
 pub enum TinyVec<A: Array> {
   #[allow(missing_docs)]
@@ -60,22 +96,6 @@ impl<A: Array + Default> Default for TinyVec<A> {
     TinyVec::Inline(ArrayVec::default())
   }
 }
-impl<A: Array> TinyVec<A> {
-  /// Moves the content of the TinyVec to the heap, if it's inline.
-  #[allow(clippy::missing_inline_in_public_items)]
-  pub fn move_to_the_heap(&mut self) {
-    match self {
-      TinyVec::Inline(ref mut arr) => {
-        let mut v = Vec::with_capacity(A::CAPACITY * 2);
-        for item in arr.drain(..) {
-          v.push(item);
-        }
-        replace(self, TinyVec::Heap(v));
-      }
-      TinyVec::Heap(_) => (),
-    }
-  }
-}
 
 impl<A: Array> Deref for TinyVec<A> {
   type Target = [A::Item];
@@ -117,6 +137,23 @@ impl<A: Array, I: SliceIndex<[A::Item]>> IndexMut<I> for TinyVec<A> {
   }
 }
 
+impl<A: Array> TinyVec<A> {
+  /// Moves the content of the TinyVec to the heap, if it's inline.
+  #[allow(clippy::missing_inline_in_public_items)]
+  pub fn move_to_the_heap(&mut self) {
+    match self {
+      TinyVec::Inline(ref mut arr) => {
+        let mut v = Vec::with_capacity(A::CAPACITY * 2);
+        for item in arr.drain(..) {
+          v.push(item);
+        }
+        replace(self, TinyVec::Heap(v));
+      }
+      TinyVec::Heap(_) => (),
+    }
+  }
+}
+
 impl<A: Array> TinyVec<A> {
   /// Move all values from `other` into this vec.
   #[inline]
@@ -275,8 +312,6 @@ impl<A: Array> TinyVec<A> {
     }
   }
 
-  // LATER(Vec): drain_filter #nightly https://github.com/rust-lang/rust/issues/43244
-
   /// Clone each element of the slice into this vec.
   #[inline]
   pub fn extend_from_slice(&mut self, sli: &[A::Item])
@@ -307,6 +342,20 @@ impl<A: Array> TinyVec<A> {
     }
   }
 
+  #[inline(always)]
+  #[doc(hidden)] // Internal implementation details of `tiny_vec!`
+  pub fn from_either_with_capacity(
+    cap: usize,
+    make_array: impl FnOnce() -> ArrayVec<A>,
+    make_vec: impl FnOnce() -> Vec<A::Item>,
+  ) -> Self {
+    if cap <= A::CAPACITY {
+      TinyVec::Inline(make_array())
+    } else {
+      TinyVec::Heap(make_vec())
+    }
+  }
+
   /// Inserts an item at the position given, moving all following elements +1
   /// index.
   ///
@@ -418,8 +467,6 @@ impl<A: Array> TinyVec<A> {
     }
   }
 
-  // NIGHTLY: remove_item, https://github.com/rust-lang/rust/issues/40062
-
   /// Resize the vec to the new length.
   ///
   /// If it needs to be longer, it's filled with clones of the provided value.
@@ -505,8 +552,6 @@ impl<A: Array> TinyVec<A> {
     }
   }
 
-  // LATER(Vec): splice
-
   /// Splits the collection at the point given.
   ///
   /// * `[0, at)` stays in this vec
@@ -594,8 +639,6 @@ pub struct TinyVecDrain<'p, A: Array> {
   target_index: usize,
   target_count: usize,
 }
-// GoodFirstIssue: this entire type is correct but slow.
-// NIGHTLY: vec_drain_as_slice, https://github.com/rust-lang/rust/issues/58957
 impl<'p, A: Array> Iterator for TinyVecDrain<'p, A> {
   type Item = A::Item;
   #[inline]
@@ -665,6 +708,37 @@ impl<A: Array> From<ArrayVec<A>> for TinyVec<A> {
   }
 }
 
+impl<T, A> From<&'_ [T]> for TinyVec<A>
+where
+  T: Clone + Default,
+  A: Array<Item = T> + Default,
+{
+  #[inline]
+  #[must_use]
+  fn from(slice: &[T]) -> Self {
+    if slice.len() > A::CAPACITY {
+      TinyVec::Heap(slice.into())
+    } else {
+      let mut arr = ArrayVec::new();
+      arr.extend_from_slice(slice);
+
+      TinyVec::Inline(arr)
+    }
+  }
+}
+
+impl<T, A> From<&'_ mut [T]> for TinyVec<A>
+where
+  T: Clone + Default,
+  A: Array<Item = T> + Default,
+{
+  #[inline]
+  #[must_use]
+  fn from(slice: &mut [T]) -> Self {
+    Self::from(&*slice)
+  }
+}
+
 impl<A: Array + Default> FromIterator<A::Item> for TinyVec<A> {
   #[inline]
   #[must_use]
@@ -684,6 +758,19 @@ pub enum TinyVecIterator<A: Array> {
   #[allow(missing_docs)]
   Heap(alloc::vec::IntoIter<A::Item>),
 }
+
+impl<A: Array> TinyVecIterator<A> {
+  /// Returns the remaining items of this iterator as a slice.
+  #[inline]
+  #[must_use]
+  pub fn as_slice(&self) -> &[A::Item] {
+    match self {
+      TinyVecIterator::Inline(a) => a.as_slice(),
+      TinyVecIterator::Heap(v) => v.as_slice(),
+    }
+  }
+}
+
 impl<A: Array> Iterator for TinyVecIterator<A> {
   type Item = A::Item;
   #[inline]
@@ -724,6 +811,13 @@ impl<A: Array> Iterator for TinyVecIterator<A> {
   }
 }
 
+impl<A: Array> Debug for TinyVecIterator<A> where A::Item: Debug {
+  #[allow(clippy::missing_inline_in_public_items)]
+  fn fmt(&self, f: &mut Formatter<'_>) -> core::fmt::Result {
+    f.debug_tuple("TinyVecIterator").field(&self.as_slice()).finish()
+  }
+}
+
 impl<A: Array> IntoIterator for TinyVec<A> {
   type Item = A::Item;
   type IntoIter = TinyVecIterator<A>;
@@ -737,6 +831,26 @@ impl<A: Array> IntoIterator for TinyVec<A> {
   }
 }
 
+impl<'a, A: Array> IntoIterator for &'a mut TinyVec<A> {
+  type Item = &'a mut A::Item;
+  type IntoIter = alloc::slice::IterMut<'a, A::Item>;
+  #[inline(always)]
+  #[must_use]
+  fn into_iter(self) -> Self::IntoIter {
+    self.iter_mut()
+  }
+}
+
+impl<'a, A: Array> IntoIterator for &'a TinyVec<A> {
+  type Item = &'a A::Item;
+  type IntoIter = alloc::slice::Iter<'a, A::Item>;
+  #[inline(always)]
+  #[must_use]
+  fn into_iter(self) -> Self::IntoIter {
+    self.iter()
+  }
+}
+
 impl<A: Array> PartialEq for TinyVec<A>
 where
   A::Item: PartialEq,
@@ -744,7 +858,7 @@ where
   #[inline]
   #[must_use]
   fn eq(&self, other: &Self) -> bool {
-    self.deref().eq(other.deref())
+    self.as_slice().eq(other.as_slice())
   }
 }
 impl<A: Array> Eq for TinyVec<A> where A::Item: Eq {}
@@ -756,7 +870,7 @@ where
   #[inline]
   #[must_use]
   fn partial_cmp(&self, other: &Self) -> Option<core::cmp::Ordering> {
-    self.deref().partial_cmp(other.deref())
+    self.as_slice().partial_cmp(other.as_slice())
   }
 }
 impl<A: Array> Ord for TinyVec<A>
@@ -766,7 +880,7 @@ where
   #[inline]
   #[must_use]
   fn cmp(&self, other: &Self) -> core::cmp::Ordering {
-    self.deref().cmp(other.deref())
+    self.as_slice().cmp(other.as_slice())
   }
 }
 
@@ -777,7 +891,7 @@ where
   #[inline]
   #[must_use]
   fn eq(&self, other: &&A) -> bool {
-    self.as_slice() == other.as_slice()
+    self.as_slice().eq(other.as_slice())
   }
 }
 
@@ -788,7 +902,7 @@ where
   #[inline]
   #[must_use]
   fn eq(&self, other: &&[A::Item]) -> bool {
-    self.deref() == *other
+    self.as_slice().eq(*other)
   }
 }
 
@@ -802,9 +916,9 @@ where
   }
 }
 
-// //
+// // // // // // // //
 // Formatting impls
-// //
+// // // // // // // //
 
 impl<A: Array> Binary for TinyVec<A>
 where
